from pathlib import Path
from unittest.mock import Mock
from typing import Dict, Text
import webbrowser

import pytest
from rasax.community.services.data_service import DataService
from rasax.community.services.story_service import StoryService
from sqlalchemy.orm import Session
from _pytest.monkeypatch import MonkeyPatch

from rasa.utils import common as rasa_utils
from rasax.community import initialise, config, utils
from rasax.community.services.settings_service import SettingsService
from rasax.community.services.user_service import UserService
from rasax.community.constants import (
    CONFIG_METRICS_ENABLED,
    CONFIG_METRICS_WELCOME_SHOWN,
    WELCOME_PAGE_URL,
)

EXAMPLE_URL = "https://example.com"


def test_create_community_user(
    settings_service: SettingsService, session: Session, user_service: UserService
):
    from rasax.community.constants import COMMUNITY_USERNAME

    # Change this to `False` for the test to make sure our test password is used
    config.LOCAL_MODE = False

    test_password = "test password"
    settings_service.save_local_password(test_password)

    initialise.create_community_user(session, Mock())

    actual = user_service._fetch_user(COMMUNITY_USERNAME)

    assert actual is not None
    assert actual.password_hash == user_service.hash_pw(test_password)

    # Change it back
    config.LOCAL_MODE = True


@pytest.mark.parametrize(
    "metrics_config, expected_url",
    [
        (
            # Metrics enabled, welcome page not shown yet
            {CONFIG_METRICS_ENABLED: True, CONFIG_METRICS_WELCOME_SHOWN: False},
            WELCOME_PAGE_URL,
        ),
        (
            # Metrics enabled, but upgrading from an older Rasa X version
            # Don't show welcome page
            {CONFIG_METRICS_ENABLED: True},
            EXAMPLE_URL,
        ),
        (
            # Metrics enabled, welcome page already shown
            {CONFIG_METRICS_ENABLED: True, CONFIG_METRICS_WELCOME_SHOWN: True},
            EXAMPLE_URL,
        ),
        (
            # Metrics not enabled
            {CONFIG_METRICS_ENABLED: False},
            EXAMPLE_URL,
        ),
    ],
)
def test_open_web_browser(
    monkeypatch: MonkeyPatch, metrics_config: Dict[Text, bool], expected_url: Text
):
    open_new_tab = Mock()
    monkeypatch.setattr(webbrowser, "open_new_tab", open_new_tab)

    read_global_config_value = Mock()
    read_global_config_value.return_value = metrics_config
    monkeypatch.setattr(
        rasa_utils, "read_global_config_value", read_global_config_value
    )

    write_global_config_value = Mock()
    monkeypatch.setattr(
        rasa_utils, "write_global_config_value", write_global_config_value
    )

    initialise._open_web_browser(EXAMPLE_URL)

    # Check which URL was opened first
    assert open_new_tab.call_args_list[0][0][0] == expected_url


async def test_inject_files_from_disk_does_not_dump(
    tmpdir: Path, new_project: Text, session: Session, monkeypatch: MonkeyPatch
):
    from rasa.cli import scaffold

    # Create Rasa project
    project_path = str(tmpdir)
    scaffold.create_initial_project(project_path)

    dump = Mock()

    monkeypatch.setattr(utils, "should_dump", lambda: True)
    monkeypatch.setattr(SettingsService, "_dump_config", dump)
    monkeypatch.setattr(DataService, "_dump_data", dump)
    monkeypatch.setattr(StoryService, "_dump_stories", dump)
    await initialise.inject_files_from_disk(project_path, "data", session)

    dump.assert_not_called()
